import pyhausbus.HausBusUtils as HausBusUtils

class GetMinBrightness:
  CLASS_ID = 21
  FUNCTION_ID = 7

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetMinBrightness()

  def __str__(self):
    return f"GetMinBrightness()"






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils

class PCServer(ABusFeature):
  CLASS_ID:int = 1

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return PCServer(HausBusUtils.getObjectId(deviceId, 1, instanceId))

  """
  @param command .
  """
  def exec(self, command:str):
    LOGGER.debug("exec"+" command = "+str(command))
    hbCommand = HausBusCommand(self.objectId, 0, "exec")
    hbCommand.addString(command)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param varId .
  @param varValue .
  """
  def setVariable(self, varId:int, varValue:int):
    LOGGER.debug("setVariable"+" varId = "+str(varId)+" varValue = "+str(varValue))
    hbCommand = HausBusCommand(self.objectId, 126, "setVariable")
    hbCommand.addByte(varId)
    hbCommand.addByte(varValue)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def shutdown(self):
    LOGGER.debug("shutdown")
    hbCommand = HausBusCommand(self.objectId, 11, "shutdown")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def restart(self):
    LOGGER.debug("restart")
    hbCommand = HausBusCommand(self.objectId, 12, "restart")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def quit(self):
    LOGGER.debug("quit")
    hbCommand = HausBusCommand(self.objectId, 20, "quit")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evOnline(self):
    LOGGER.debug("evOnline")
    hbCommand = HausBusCommand(self.objectId, 200, "evOnline")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evOffline(self):
    LOGGER.debug("evOffline")
    hbCommand = HausBusCommand(self.objectId, 201, "evOffline")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def standby(self):
    LOGGER.debug("standby")
    hbCommand = HausBusCommand(self.objectId, 10, "standby")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def reloadUserPlugin(self):
    LOGGER.debug("reloadUserPlugin")
    hbCommand = HausBusCommand(self.objectId, 13, "reloadUserPlugin")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import pyhausbus.HausBusUtils as HausBusUtils

class EvOff:
  CLASS_ID = 33
  FUNCTION_ID = 200

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvOff()

  def __str__(self):
    return f"EvOff()"






import pyhausbus.HausBusUtils as HausBusUtils

class Blink:
  CLASS_ID = 20
  FUNCTION_ID = 4

  def __init__(self,brightness:int, offTime:int, onTime:int, quantity:int):
    self.brightness=brightness
    self.offTime=offTime
    self.onTime=onTime
    self.quantity=quantity


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Blink(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"Blink(brightness={self.brightness}, offTime={self.offTime}, onTime={self.onTime}, quantity={self.quantity})"

  '''
  @param brightness 0-100% Helligkeit.
  '''
  def getBrightness(self):
    return self.brightness

  '''
  @param offTime a 100ms.
  '''
  def getOffTime(self):
    return self.offTime

  '''
  @param onTime a 100ms.
  '''
  def getOnTime(self):
    return self.onTime

  '''
  @param quantity Anzahl Blinks.
  '''
  def getQuantity(self):
    return self.quantity






from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.EFirmwareId import EFirmwareId
import pyhausbus.HausBusUtils as HausBusUtils

class ModuleId:
  CLASS_ID = 0
  FUNCTION_ID = 128

  def __init__(self,name:str, size:int, majorRelease:int, minorRelease:int, firmwareId:EFirmwareId):
    self.name=name
    self.size=size
    self.majorRelease=majorRelease
    self.minorRelease=minorRelease
    self.firmwareId=firmwareId


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return ModuleId(HausBusUtils.bytesToString(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), EFirmwareId._fromBytes(dataIn, offset))

  def __str__(self):
    return f"ModuleId(name={self.name}, size={self.size}, majorRelease={self.majorRelease}, minorRelease={self.minorRelease}, firmwareId={self.firmwareId})"

  '''
  @param name Modulname.
  '''
  def getName(self):
    return self.name

  '''
  @param size Modulgroesse in Bytes.
  '''
  def getSize(self):
    return self.size

  '''
  @param majorRelease Release-Kennung Format major.minor.
  '''
  def getMajorRelease(self):
    return self.majorRelease

  '''
  @param minorRelease Release-Kennung Format major.minor.
  '''
  def getMinorRelease(self):
    return self.minorRelease

  '''
  @param firmwareId Firmware-Kennung.
  '''
  def getFirmwareId(self):
    return self.firmwareId






import pyhausbus.HausBusUtils as HausBusUtils
class MMode:

  def setIncrement(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 0, self.value)
    return self;

  def isIncrement(self):
    return HausBusUtils.isBitSet(0, self.value)
  def setFallingEdge(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 1, self.value)
    return self;

  def isFallingEdge(self):
    return HausBusUtils.isBitSet(1, self.value)
  def setRisingEdge(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 2, self.value)
    return self;

  def isRisingEdge(self):
    return HausBusUtils.isBitSet(2, self.value)
  def setActiveLow(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 3, self.value)
    return self;

  def isActiveLow(self):
    return HausBusUtils.isBitSet(3, self.value)
  def setReserved1(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 4, self.value)
    return self;

  def isReserved1(self):
    return HausBusUtils.isBitSet(4, self.value)
  def setReserved2(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 5, self.value)
    return self;

  def isReserved2(self):
    return HausBusUtils.isBitSet(5, self.value)
  def setReserved3(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 6, self.value)
    return self;

  def isReserved3(self):
    return HausBusUtils.isBitSet(6, self.value)
  def setReserved4(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 7, self.value)
    return self;

  def isReserved4(self):
    return HausBusUtils.isBitSet(7, self.value)
  def __init__(self, value:int):
    self.value = value

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    return MMode(HausBusUtils.bytesToInt(data, offset))



  def getValue(self):
    return self.value
  def getEntryNames(self):
    result = []
    result.append("Increment")
    result.append("FallingEdge")
    result.append("RisingEdge")
    result.append("ActiveLow")
    result.append("Reserved1")
    result.append("Reserved2")
    result.append("Reserved3")
    result.append("Reserved4")
    return result
  def setEntry(self,name:str, setValue:bool):
    if (name == "Increment"):
      self.setIncrement(setValue)
    if (name == "FallingEdge"):
      self.setFallingEdge(setValue)
    if (name == "RisingEdge"):
      self.setRisingEdge(setValue)
    if (name == "ActiveLow"):
      self.setActiveLow(setValue)
    if (name == "Reserved1"):
      self.setReserved1(setValue)
    if (name == "Reserved2"):
      self.setReserved2(setValue)
    if (name == "Reserved3"):
      self.setReserved3(setValue)
    if (name == "Reserved4"):
      self.setReserved4(setValue)

  def __str__(self):
    return f"MMode(Increment = {self.isIncrement()}, FallingEdge = {self.isFallingEdge()}, RisingEdge = {self.isRisingEdge()}, ActiveLow = {self.isActiveLow()}, Reserved1 = {self.isReserved1()}, Reserved2 = {self.isReserved2()}, Reserved3 = {self.isReserved3()}, Reserved4 = {self.isReserved4()})"






import pyhausbus.HausBusUtils as HausBusUtils

class EvBright:
  CLASS_ID = 39
  FUNCTION_ID = 202

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvBright()

  def __str__(self):
    return f"EvBright()"






from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.EType import EType
import pyhausbus.HausBusUtils as HausBusUtils

class GetSystemVariable:
  CLASS_ID = 0
  FUNCTION_ID = 17

  def __init__(self,type:EType, index:int):
    self.type=type
    self.index=index


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetSystemVariable(EType._fromBytes(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"GetSystemVariable(type={self.type}, index={self.index})"

  '''
  @param type Hier wird der Typ der Variable.
  '''
  def getType(self):
    return self.type

  '''
  @param index Die Variablen liegen mehrfach vor 32xBIT.
  '''
  def getIndex(self):
    return self.index






import pyhausbus.HausBusUtils as HausBusUtils

class RulesData:
  CLASS_ID = 0
  FUNCTION_ID = 134

  def __init__(self,offset:int, data:bytearray):
    self.offset=offset
    self.data=data


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return RulesData(HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToBlob(dataIn, offset))

  def __str__(self):
    return f"RulesData(offset={self.offset}, data={self.data})"

  '''
  @param offset offset im Gesamtregelblock.
  '''
  def getOffset(self):
    return self.offset

  '''
  @param data .
  '''
  def getData(self):
    return self.data






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EErrorCode(Enum):
  PIN0_FUNCTION_NOT_SUPPORTED=16
  PIN1_FUNCTION_NOT_SUPPORTED=17
  PIN2_FUNCTION_NOT_SUPPORTED=18
  PIN3_FUNCTION_NOT_SUPPORTED=19
  PIN4_FUNCTION_NOT_SUPPORTED=20
  PIN5_FUNCTION_NOT_SUPPORTED=21
  PIN6_FUNCTION_NOT_SUPPORTED=22
  PIN7_FUNCTION_NOT_SUPPORTED=23
  PIN0_FUNCTION_NOT_ACTIVATED=24
  PIN1_FUNCTION_NOT_ACTIVATED=25
  PIN2_FUNCTION_NOT_ACTIVATED=26
  PIN3_FUNCTION_NOT_ACTIVATED=27
  PIN4_FUNCTION_NOT_ACTIVATED=28
  PIN5_FUNCTION_NOT_ACTIVATED=29
  PIN6_FUNCTION_NOT_ACTIVATED=30
  PIN7_FUNCTION_NOT_ACTIVATED=31
  CONFIGURATION_OUT_OF_MEMORY=128
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EErrorCode.__members__.values():
      if (act.value == checkValue):
        return act

    return EErrorCode.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EErrorCode':
    try:
      return EErrorCode[name]
    except KeyError:
      return EErrorCode.SER_UNKNOWN 







from pyhausbus.de.hausbus.homeassistant.proxy.schalter.params.EState import EState
import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 19
  FUNCTION_ID = 129

  def __init__(self,state:EState, duration:int, offTime:int, onTime:int):
    self.state=state
    self.duration=duration
    self.offTime=offTime
    self.onTime=onTime


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(EState._fromBytes(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"Status(state={self.state}, duration={self.duration}, offTime={self.offTime}, onTime={self.onTime})"

  '''
  @param state .
  '''
  def getState(self):
    return self.state

  '''
  @param duration Einschaltdauer: Wert * Zeitbasis [ms]\r\n0=Endlos\r\nWenn state TOGGLE.
  '''
  def getDuration(self):
    return self.duration

  '''
  @param offTime Dauer der Aus-Phase beim Togglen: \r\nWert * Zeitbasis [ms].
  '''
  def getOffTime(self):
    return self.offTime

  '''
  @param onTime Dauer der An-Phase beim Togglen: \r\nWert * Zeitbasis [ms].
  '''
  def getOnTime(self):
    return self.onTime






from pyhausbus.de.hausbus.homeassistant.proxy.rollladen.params.ENewState import ENewState
import pyhausbus.HausBusUtils as HausBusUtils

class EvNewMainState:
  CLASS_ID = 18
  FUNCTION_ID = 251

  def __init__(self,newState:ENewState, preState:ENewState):
    self.newState=newState
    self.preState=preState


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvNewMainState(ENewState._fromBytes(dataIn, offset), ENewState._fromBytes(dataIn, offset))

  def __str__(self):
    return f"EvNewMainState(newState={self.newState}, preState={self.preState})"

  '''
  @param newState State.
  '''
  def getNewState(self):
    return self.newState

  '''
  @param preState State.
  '''
  def getPreState(self):
    return self.preState






import pyhausbus.HausBusUtils as HausBusUtils

class SetSignalCount:
  CLASS_ID = 90
  FUNCTION_ID = 2

  def __init__(self,signalCount:int):
    self.signalCount=signalCount


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetSignalCount(HausBusUtils.bytesToDWord(dataIn, offset))

  def __str__(self):
    return f"SetSignalCount(signalCount={self.signalCount})"

  '''
  @param signalCount .
  '''
  def getSignalCount(self):
    return self.signalCount






import pyhausbus.HausBusUtils as HausBusUtils

class EvSignal:
  CLASS_ID = 90
  FUNCTION_ID = 200

  def __init__(self,time:int, signalCount:int, power:int, signalDuration:int):
    self.time=time
    self.signalCount=signalCount
    self.power=power
    self.signalDuration=signalDuration


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvSignal(HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset))

  def __str__(self):
    return f"EvSignal(time={self.time}, signalCount={self.signalCount}, power={self.power}, signalDuration={self.signalDuration})"

  '''
  @param time Systemzeit des ESP zu Debugzwecken.
  '''
  def getTime(self):
    return self.time

  '''
  @param signalCount Anzahl gez?hlter S0 Signale seit dem letzten Zur?cksetzen.
  '''
  def getSignalCount(self):
    return self.signalCount

  '''
  @param power Aktuelle Leistung in Watt.
  '''
  def getPower(self):
    return self.power

  '''
  @param signalDuration Dauer des gemessenen S0 Signals in ms.
  '''
  def getSignalDuration(self):
    return self.signalDuration






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EDirection(Enum):
  TOGGLE=0
  TO_DARK=255
  TO_LIGHT=1
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EDirection.__members__.values():
      if (act.value == checkValue):
        return act

    return EDirection.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EDirection':
    try:
      return EDirection[name]
    except KeyError:
      return EDirection.SER_UNKNOWN 







from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.pCA9555.params.EErrorCode import EErrorCode

class PCA9555(ABusFeature):
  CLASS_ID:int = 165

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return PCA9555(HausBusUtils.getObjectId(deviceId, 165, instanceId))

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import os
from collections import defaultdict
from typing import List, Dict, Optional
from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.EFirmwareId import EFirmwareId
from pyhausbus.HausBusUtils import LOGGER

def load_file(path: str) -> list[str]:
    try:
        with open(path, "r", encoding="latin1") as file:
            return [line.strip() for line in file if line.strip()]
    except FileNotFoundError:
        LOGGER.error(f"Datei nicht gefunden: {path}")
        return []
    except Exception as e:
        LOGGER.error(f"Fehler beim Laden der Datei {path}: {e}")
        return []

class Templates:
    template_root_dir = os.path.join(os.path.dirname(__file__), "templates")
    LOGGER.debug(f"templateRootDir = {template_root_dir}")
    filter_non_existing = True
    _instance = None
    class_mappings: Dict[str, str] = {}

    def __init__(self):
        self.module_types: Dict['ModuleType', str] = {}
        self.feature_names: Dict['ModuleType', List['FeatureEntry']] = {}

        try:
            lines = load_file(os.path.join(self.template_root_dir, "deviceTypes.def"))
            for line in lines:
                tokens = line.split(",")
                firmware_id = EFirmwareId.value_of(tokens[0])
                fcke = int(tokens[1].replace("0x", ""), 16)
                name = tokens[2]
                self.module_types[ModuleType(firmware_id, fcke)] = name


            root_dir = self.template_root_dir
            if not os.path.exists(root_dir):
                root_dir = "../JavaLib/" + self.template_root_dir

            for act_file in os.listdir(root_dir):
                if act_file.endswith(".tpl"):
                    tokens = act_file.split("_")
                    firmware_id = EFirmwareId.value_of(tokens[0])
                    fcke = int(tokens[1], 16)
                    features = []
                    self.feature_names[ModuleType(firmware_id, fcke)] = features

                    lines = load_file(os.path.join(root_dir, act_file))
                    for line in lines:
                        if line.strip():
                            features.append(FeatureEntry(line))

            '''for module_type, features in self.feature_names.items():
              print(module_type)
              for entry in features:
                print(entry)
              
            quit()'''

            lines = load_file(os.path.join(self.template_root_dir, "classMapping.def"))
            if lines:
                for line in lines:
                    tokens = line.split(",")
                    orig_name = tokens[0]
                    mapped_name = tokens[1]
                    Templates.class_mappings[orig_name] = mapped_name
            LOGGER.debug(f"module types = {len(self.module_types)}")
            LOGGER.debug(f"feature names = {len(self.feature_names)}")
        except Exception as e:
            LOGGER.error(f"Fehler beim Initialisieren der Templates: {e}")

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = Templates()
        return cls._instance

    def get_feature_name_from_template(self, firmware_id, fcke, class_id, instance_id) -> Optional[str]:
        if fcke != -1:
            ''' print("get_feature_name_from_template firmware_id = ",firmware_id," fcke = ",fcke," class_id = ",class_id," instance_id = ",instance_id)'''
            features = self.get_features(firmware_id, fcke)

            if features:
                for entry in features:
                    if entry.suits(class_id, instance_id):
                        return entry.name
        return None

    def getModuleName(self, firmwareId: EFirmwareId, fcke: int):
      for entry, name in self.module_types.items():
        if entry.suits(firmwareId, fcke):
            return name
      return None
    
    def get_features(self, firmware_id, fcke) -> Optional[List['FeatureEntry']]:
        for module_type, features in self.feature_names.items():
            if module_type.suits(firmware_id, fcke):
                return list(features)
        return None


class FeatureEntry:
    def __init__(self, line: str):
        tokens = line.split(",")
        self.class_id = int(tokens[0])
        self.instance_id = int(tokens[1])
        self.name = tokens[2]
        self.loxone_aktor_type = tokens[3] if len(tokens) > 3 else ""

    def suits(self, class_id: int, instance_id: int) -> bool:
        return self.class_id == class_id and self.instance_id == instance_id

    def __str__(self):
        return f"FeatureEntry(classId={self.class_id}, instanceId={self.instance_id}, name='{self.name}', loxoneAktorType='{self.loxone_aktor_type}')"


class ModuleType:
    def __init__(self, firmware_id, fcke):
        self.firmware_id = firmware_id
        self.fcke = fcke

    def suits(self, check_firmware_id, check_fcke) -> bool:
        if check_fcke != self.fcke:
            return False
        if check_firmware_id == self.firmware_id:
            return True
        if str(check_firmware_id).startswith("HB") and str(self.firmware_id).startswith("HB"):
            return True
        return False

    def __eq__(self, other):
        if not isinstance(other, ModuleType):
            return False
        return self.firmware_id == other.firmware_id and self.fcke == other.fcke

    def __hash__(self):
        return hash((self.firmware_id, self.fcke))

    def __str__(self):
        return f"ModuleType(firmwareId={self.firmware_id}, fcke={self.fcke})"



from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.EState import EState
from pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.MEventMask import MEventMask
from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.MOptionMask import MOptionMask
from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.EErrorCode import EErrorCode
from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.EEnable import EEnable
from pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Status import Status
from pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Enabled import Enabled

class Taster(ABusFeature):
  CLASS_ID:int = 16

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return Taster(HausBusUtils.getObjectId(deviceId, 16, instanceId))

  """
  @param state .
  """
  def evClicked(self, state:EState):
    LOGGER.debug("evClicked"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 201, "evClicked")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  """
  def evDoubleClick(self, state:EState):
    LOGGER.debug("evDoubleClick"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 202, "evDoubleClick")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  """
  def evHoldStart(self, state:EState):
    LOGGER.debug("evHoldStart"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 203, "evHoldStart")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  """
  def evHoldEnd(self, state:EState):
    LOGGER.debug("evHoldEnd"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 204, "evHoldEnd")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param holdTimeout Zeit a 10ms.
  @param waitForDoubleClickTimeout Zeit a 10ms.
  @param eventMask Jedes gesetzte Bit aktiviert das Melden des entsprechenden Events..
  @param optionMask 0: invertiert die Eingangslogik\r\n1: setzt den Initialzustand auf 0.
  @param debounceTime EntprellZeit in ms 1-254\r\nStandard ist 40ms.
  """
  def setConfiguration(self, holdTimeout:int, waitForDoubleClickTimeout:int, eventMask:MEventMask, optionMask:MOptionMask, debounceTime:int):
    LOGGER.debug("setConfiguration"+" holdTimeout = "+str(holdTimeout)+" waitForDoubleClickTimeout = "+str(waitForDoubleClickTimeout)+" eventMask = "+str(eventMask)+" optionMask = "+str(optionMask)+" debounceTime = "+str(debounceTime))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(holdTimeout)
    hbCommand.addByte(waitForDoubleClickTimeout)
    hbCommand.addByte(eventMask.getValue())
    hbCommand.addByte(optionMask.getValue())
    hbCommand.addByte(debounceTime)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param holdTimeout Zeit a 10ms.
  @param waitForDoubleClickTimeout Zeit a 10ms.
  @param eventMask Jedes gesetzte Bit aktiviert das Melden des entsprechenden Events..
  @param optionMask 0: invertiert die Eingangslogik\r\n1: setzt den Initialzustand auf 0.
  @param debounceTime EntprellZeit in ms 1-254\r\nStandard ist 40ms.
  """
  def Configuration(self, holdTimeout:int, waitForDoubleClickTimeout:int, eventMask:MEventMask, optionMask:MOptionMask, debounceTime:int):
    LOGGER.debug("Configuration"+" holdTimeout = "+str(holdTimeout)+" waitForDoubleClickTimeout = "+str(waitForDoubleClickTimeout)+" eventMask = "+str(eventMask)+" optionMask = "+str(optionMask)+" debounceTime = "+str(debounceTime))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(holdTimeout)
    hbCommand.addByte(waitForDoubleClickTimeout)
    hbCommand.addByte(eventMask.getValue())
    hbCommand.addByte(optionMask.getValue())
    hbCommand.addByte(debounceTime)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  """
  def evCovered(self, state:EState):
    LOGGER.debug("evCovered"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 200, "evCovered")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  """
  def evFree(self, state:EState):
    LOGGER.debug("evFree"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 205, "evFree")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param enable FALSE: Deaktiviert das Versenden von Events\r\nTRUE: Aktiviert das Versenden von Events\r\nINVERT: Invertiert das aktuelle Verhalten.
  @param disabledDuration Zeit1s-255s f?  ? ? ?r die die Events deaktiviert werden sollen 0 = unendlich \r\nDieser Parameter wirkt nur.
  """
  def enableEvents(self, enable:EEnable, disabledDuration:int):
    LOGGER.debug("enableEvents"+" enable = "+str(enable)+" disabledDuration = "+str(disabledDuration))
    hbCommand = HausBusCommand(self.objectId, 2, "enableEvents")
    hbCommand.addByte(enable.value)
    hbCommand.addByte(disabledDuration)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getStatus(self):
    LOGGER.debug("getStatus")
    hbCommand = HausBusCommand(self.objectId, 3, "getStatus")
    ResultWorker()._setResultInfo(Status,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  """
  def Status(self, state:EState):
    LOGGER.debug("Status"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 129, "Status")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param enabled 0: Events wurden gerade deaktiviert\r\n1: Events wurden gerade aktiviert.
  """
  def evEnabled(self, enabled:int):
    LOGGER.debug("evEnabled"+" enabled = "+str(enabled))
    hbCommand = HausBusCommand(self.objectId, 206, "evEnabled")
    hbCommand.addByte(enabled)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param enabled 0: Events sind deaktviert\r\n1: Events sind aktiviert.
  """
  def Enabled(self, enabled:int):
    LOGGER.debug("Enabled"+" enabled = "+str(enabled))
    hbCommand = HausBusCommand(self.objectId, 130, "Enabled")
    hbCommand.addByte(enabled)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getEnabled(self):
    LOGGER.debug("getEnabled")
    hbCommand = HausBusCommand(self.objectId, 4, "getEnabled")
    ResultWorker()._setResultInfo(Enabled,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def triggerStatusEvent(self):
    LOGGER.debug("triggerStatusEvent")
    hbCommand = HausBusCommand(self.objectId, 5, "triggerStatusEvent")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





from pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.params.ELastEvent import ELastEvent
import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 42
  FUNCTION_ID = 129

  def __init__(self,celsius:int, centiCelsius:int, lastEvent:ELastEvent):
    self.celsius=celsius
    self.centiCelsius=centiCelsius
    self.lastEvent=lastEvent


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(HausBusUtils.bytesToSInt(dataIn, offset), HausBusUtils.bytesToSInt(dataIn, offset), ELastEvent._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Status(celsius={self.celsius}, centiCelsius={self.centiCelsius}, lastEvent={self.lastEvent})"

  '''
  @param celsius Grad Celsius.
  '''
  def getCelsius(self):
    return self.celsius

  '''
  @param centiCelsius hundertstel Grad Celsius.
  '''
  def getCentiCelsius(self):
    return self.centiCelsius

  '''
  @param lastEvent .
  '''
  def getLastEvent(self):
    return self.lastEvent






from pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.params.ELastEvent import ELastEvent
import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 41
  FUNCTION_ID = 129

  def __init__(self,power:int, centiPower:int, lastEvent:ELastEvent):
    self.power=power
    self.centiPower=centiPower
    self.lastEvent=lastEvent


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(HausBusUtils.bytesToSInt(dataIn, offset), HausBusUtils.bytesToSInt(dataIn, offset), ELastEvent._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Status(power={self.power}, centiPower={self.centiPower}, lastEvent={self.lastEvent})"

  '''
  @param power Aktueller Stromverbrauch.
  '''
  def getPower(self):
    return self.power

  '''
  @param centiPower hundertstel Stromverbrauch.
  '''
  def getCentiPower(self):
    return self.centiPower

  '''
  @param lastEvent .
  '''
  def getLastEvent(self):
    return self.lastEvent






import pyhausbus.HausBusUtils as HausBusUtils

class Restart:
  CLASS_ID = 1
  FUNCTION_ID = 12

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Restart()

  def __str__(self):
    return f"Restart()"






import pyhausbus.HausBusUtils as HausBusUtils

class EvCurrent:
  CLASS_ID = 90
  FUNCTION_ID = 201

  def __init__(self,current:int):
    self.current=current


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvCurrent(HausBusUtils.bytesToDWord(dataIn, offset))

  def __str__(self):
    return f"EvCurrent(current={self.current})"

  '''
  @param current Verbrauchter Strom in Wattstunden.
  '''
  def getCurrent(self):
    return self.current






import pyhausbus.HausBusUtils as HausBusUtils

class ClearSignalCount:
  CLASS_ID = 90
  FUNCTION_ID = 7

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return ClearSignalCount()

  def __str__(self):
    return f"ClearSignalCount()"






import pyhausbus.HausBusUtils as HausBusUtils

class TimeDifference:
  CLASS_ID = 0
  FUNCTION_ID = 197

  def __init__(self,timeDifference:int):
    self.timeDifference=timeDifference


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return TimeDifference(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"TimeDifference(timeDifference={self.timeDifference})"

  '''
  @param timeDifference Abweichung der internen Wochenzeit in Minuten Achtung: Vorzeichenbehaftetes Byte. 255 entspricht -1.
  '''
  def getTimeDifference(self):
    return self.timeDifference






from pyhausbus.de.hausbus.homeassistant.proxy.systemTime.params.EWeekDay import EWeekDay
from pyhausbus.de.hausbus.homeassistant.proxy.systemTime.params.EDate import EDate
from pyhausbus.de.hausbus.homeassistant.proxy.systemTime.params.EMonth import EMonth
import pyhausbus.HausBusUtils as HausBusUtils

class SetTime:
  CLASS_ID = 3
  FUNCTION_ID = 1

  def __init__(self,weekDay:EWeekDay, date:EDate, month:EMonth, year:int, hours:int, minutes:int, seconds:int):
    self.weekDay=weekDay
    self.date=date
    self.month=month
    self.year=year
    self.hours=hours
    self.minutes=minutes
    self.seconds=seconds


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetTime(EWeekDay._fromBytes(dataIn, offset), EDate._fromBytes(dataIn, offset), EMonth._fromBytes(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"SetTime(weekDay={self.weekDay}, date={self.date}, month={self.month}, year={self.year}, hours={self.hours}, minutes={self.minutes}, seconds={self.seconds})"

  '''
  @param weekDay .
  '''
  def getWeekDay(self):
    return self.weekDay

  '''
  @param date .
  '''
  def getDate(self):
    return self.date

  '''
  @param month .
  '''
  def getMonth(self):
    return self.month

  '''
  @param year .
  '''
  def getYear(self):
    return self.year

  '''
  @param hours .
  '''
  def getHours(self):
    return self.hours

  '''
  @param minutes .
  '''
  def getMinutes(self):
    return self.minutes

  '''
  @param seconds .
  '''
  def getSeconds(self):
    return self.seconds






from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.MOption import MOption
import pyhausbus.HausBusUtils as HausBusUtils

class SetDebugOptions:
  CLASS_ID = 0
  FUNCTION_ID = 124

  def __init__(self,option:MOption):
    self.option=option


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetDebugOptions(MOption._fromBytes(dataIn, offset))

  def __str__(self):
    return f"SetDebugOptions(option={self.option})"

  '''
  @param option SEND_TRIGGERED_RULE_EVENT: generiert ein Event zu einer aktivierten Regel\r\nREAD_ONLY_GATEWAYS: schaltet das Versenden saemtlicher Nachrichten ab. Eingehende Nachrichten werden verarbeitet\r\nREPORT_INTERNAL_TEMPERATURE: aktiviert den internen TemperaturSensor des Prozessors ungenau\r\nSEND_ZERO_CROSS_DATA: sendet im Sekundentakt aufgezeichnete Daten zur Nulldurchganserkennung bei Dimmer-Modulen.
  '''
  def getOption(self) -> MOption:
    return self.option






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.gateway.params.EErrorCode import EErrorCode
from pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.gateway.params.MOptions import MOptions
from pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.MinIdleTime import MinIdleTime
from pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.ConnectedDevices import ConnectedDevices
from pyhausbus.de.hausbus.homeassistant.proxy.gateway.params.EValue import EValue

class Gateway(ABusFeature):
  CLASS_ID:int = 176

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return Gateway(HausBusUtils.getObjectId(deviceId, 176, instanceId))

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param options enabled: Dies Gateway ist aktiv und leitet Nachrichten weiter\r\npreferLoxone: Gateway kommuniziert bevorzugt im Loxone-Protokoll\r\nenableConsole: aktiviert das senden von Debugausgaben\r\nmaster: dieses Gateway soll das Bus-Timing verwalten\r\n\r\nReservierte Bits muessen immer deaktiviert sein. Das Aktivieren eines reservierten Bits fuehrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  """
  def setConfiguration(self, options:MOptions):
    LOGGER.debug("setConfiguration"+" options = "+str(options))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(options.getValue())
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getMinIdleTime(self):
    LOGGER.debug("getMinIdleTime")
    hbCommand = HausBusCommand(self.objectId, 3, "getMinIdleTime")
    ResultWorker()._setResultInfo(MinIdleTime,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param time_ms Mindestwartezeit [ms].
  """
  def MinIdleTime(self, time_ms:int):
    LOGGER.debug("MinIdleTime"+" time_ms = "+str(time_ms))
    hbCommand = HausBusCommand(self.objectId, 129, "MinIdleTime")
    hbCommand.addByte(time_ms)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param options enabled: Dies Gateway ist aktiv und leitet Nachrichten weiter\r\npreferLoxone: Gateway kommuniziert bevorzugt im Loxone-Protokoll\r\nenableConsole: aktiviert das senden von Debugausgaben\r\nmaster: dieses Gateway soll das Bus-Timing verwalten.
  """
  def Configuration(self, options:MOptions):
    LOGGER.debug("Configuration"+" options = "+str(options))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(options.getValue())
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param time_ms Mindestwartezeit [ms].
  """
  def setMinIdleTime(self, time_ms:int):
    LOGGER.debug("setMinIdleTime"+" time_ms = "+str(time_ms))
    hbCommand = HausBusCommand(self.objectId, 4, "setMinIdleTime")
    hbCommand.addByte(time_ms)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConnectedDevices(self):
    LOGGER.debug("getConnectedDevices")
    hbCommand = HausBusCommand(self.objectId, 5, "getConnectedDevices")
    ResultWorker()._setResultInfo(ConnectedDevices,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param deviceIds .
  """
  def ConnectedDevices(self, deviceIds):
    LOGGER.debug("ConnectedDevices"+" deviceIds = "+str(deviceIds))
    hbCommand = HausBusCommand(self.objectId, 130, "ConnectedDevices")
    hbCommand.addMap(deviceIds)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param inMessagesPerMinute Anzahl der eingehenden Nachrichten pro Minute.
  @param outMessagesPerMinute Anzahl der ausgehenden Nachrichten pro Minute.
  @param inBytesPerMinute Anzahl der Datenbytes von eingehenden Nachrichten pro Minute.
  @param outBytesPerMinute Anzahl der Datenbytes von ausgehenden Nachrichten pro Minute.
  @param messageQueueHighWater Maximale Anzahl von Nachrichten in der Warteschlange innerhalb der letzten Minute.
  """
  def evGatewayLoad(self, inMessagesPerMinute:int, outMessagesPerMinute:int, inBytesPerMinute:int, outBytesPerMinute:int, messageQueueHighWater:int):
    LOGGER.debug("evGatewayLoad"+" inMessagesPerMinute = "+str(inMessagesPerMinute)+" outMessagesPerMinute = "+str(outMessagesPerMinute)+" inBytesPerMinute = "+str(inBytesPerMinute)+" outBytesPerMinute = "+str(outBytesPerMinute)+" messageQueueHighWater = "+str(messageQueueHighWater))
    hbCommand = HausBusCommand(self.objectId, 200, "evGatewayLoad")
    hbCommand.addWord(inMessagesPerMinute)
    hbCommand.addWord(outMessagesPerMinute)
    hbCommand.addDWord(inBytesPerMinute)
    hbCommand.addDWord(outBytesPerMinute)
    hbCommand.addByte(messageQueueHighWater)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param value Diese Funktion setzt das Flag \"preferLoxone\" in der Konfiguration entsprechend Persistent..
  """
  def setPreferLoxone(self, value:EValue):
    LOGGER.debug("setPreferLoxone"+" value = "+str(value))
    hbCommand = HausBusCommand(self.objectId, 2, "setPreferLoxone")
    hbCommand.addByte(value.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import pyhausbus.HausBusUtils as HausBusUtils

class EvCold:
  CLASS_ID = 32
  FUNCTION_ID = 200

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvCold()

  def __str__(self):
    return f"EvCold()"






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EFirmwareId(Enum):
  AR8=1
  MS6=2
  SD6=3
  SD485=4
  SONOFF=5
  S0_Reader=6
  ESP=7
  HBC=8
  HBX192C3=9
  ESP32=10
  ESP32C3=11
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EFirmwareId.__members__.values():
      if (act.value == checkValue):
        return act

    return EFirmwareId.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EFirmwareId':
    try:
      return EFirmwareId[name]
    except KeyError:
      return EFirmwareId.SER_UNKNOWN 

  def getTemplateId(self) -> str:
    if (self.name.startswith("HB")):
        return "HBC"
    return self.name






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EPin(Enum):
  UNUSED=255
  TASTER=16
  SCHALTER=19
  LED=21
  OW_BUS=32
  IR_SENSOR=33
  DHT_SENSOR=34
  COUNTER=35
  ROLLO_HOCH_4=128
  ROLLO_RUNTER_4=129
  S0_READER=40
  RFID_D0=43
  RFID_D1=130
  ANALOG_IN=36
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EPin.__members__.values():
      if (act.value == checkValue):
        return act

    return EPin.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EPin':
    try:
      return EPin[name]
    except KeyError:
      return EPin.SER_UNKNOWN 







from pyhausbus.de.hausbus.homeassistant.proxy.rollladen.params.EDirection import EDirection
import pyhausbus.HausBusUtils as HausBusUtils

class Start:
  CLASS_ID = 18
  FUNCTION_ID = 3

  def __init__(self,direction:EDirection):
    self.direction=direction


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Start(EDirection._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Start(direction={self.direction})"

  '''
  @param direction .
  '''
  def getDirection(self):
    return self.direction






from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.MEventMask import MEventMask
from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.MOptionMask import MOptionMask
import pyhausbus.HausBusUtils as HausBusUtils

class SetConfiguration:
  CLASS_ID = 16
  FUNCTION_ID = 1

  def __init__(self,holdTimeout:int, waitForDoubleClickTimeout:int, eventMask:MEventMask, optionMask:MOptionMask, debounceTime:int):
    self.holdTimeout=holdTimeout
    self.waitForDoubleClickTimeout=waitForDoubleClickTimeout
    self.eventMask=eventMask
    self.optionMask=optionMask
    self.debounceTime=debounceTime


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetConfiguration(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), MEventMask._fromBytes(dataIn, offset), MOptionMask._fromBytes(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"SetConfiguration(holdTimeout={self.holdTimeout}, waitForDoubleClickTimeout={self.waitForDoubleClickTimeout}, eventMask={self.eventMask}, optionMask={self.optionMask}, debounceTime={self.debounceTime})"

  '''
  @param holdTimeout Zeit a 10ms.
  '''
  def getHoldTimeout(self):
    return self.holdTimeout

  '''
  @param waitForDoubleClickTimeout Zeit a 10ms.
  '''
  def getWaitForDoubleClickTimeout(self):
    return self.waitForDoubleClickTimeout

  '''
  @param eventMask Jedes gesetzte Bit aktiviert das Melden des entsprechenden Events..
  '''
  def getEventMask(self) -> MEventMask:
    return self.eventMask

  '''
  @param optionMask 0: invertiert die Eingangslogik\r\n1: setzt den Initialzustand auf 0.
  '''
  def getOptionMask(self) -> MOptionMask:
    return self.optionMask

  '''
  @param debounceTime EntprellZeit in ms 1-254\r\nStandard ist 40ms.
  '''
  def getDebounceTime(self):
    return self.debounceTime






import pyhausbus.HausBusUtils as HausBusUtils

class EvOn:
  CLASS_ID = 20
  FUNCTION_ID = 201

  def __init__(self,brightness:int):
    self.brightness=brightness


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvOn(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"EvOn(brightness={self.brightness})"

  '''
  @param brightness 0-100% Helligkeit.
  '''
  def getBrightness(self):
    return self.brightness






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.counter.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.counter.params.MMode import MMode
from pyhausbus.de.hausbus.homeassistant.proxy.counter.data.Status import Status
from pyhausbus.de.hausbus.homeassistant.proxy.counter.params.EErrorCode import EErrorCode

class Counter(ABusFeature):
  CLASS_ID:int = 35

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return Counter(HausBusUtils.getObjectId(deviceId, 35, instanceId))

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param mode increment: 1 = Zaehler inkrementieren.
  @param debounceTime 0 - 255[ms].
  @param reportTime Zeitintervall in Minuten nach dem der Zaehler den aktuellen Stand meldet.
  @param scaleFaktor Anzahl Impulse pro Einheit z.B. pro kWh.
  """
  def setConfiguration(self, mode:MMode, debounceTime:int, reportTime:int, scaleFaktor:int):
    LOGGER.debug("setConfiguration"+" mode = "+str(mode)+" debounceTime = "+str(debounceTime)+" reportTime = "+str(reportTime)+" scaleFaktor = "+str(scaleFaktor))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(mode.getValue())
    hbCommand.addByte(debounceTime)
    hbCommand.addWord(reportTime)
    hbCommand.addWord(scaleFaktor)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getStatus(self):
    LOGGER.debug("getStatus")
    hbCommand = HausBusCommand(self.objectId, 2, "getStatus")
    ResultWorker()._setResultInfo(Status,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param counter Zaehler ganze Einheiten.
  @param fraction Bruchteil 0-scaleFactor.
  """
  def setCount(self, counter:int, fraction:int):
    LOGGER.debug("setCount"+" counter = "+str(counter)+" fraction = "+str(fraction))
    hbCommand = HausBusCommand(self.objectId, 3, "setCount")
    hbCommand.addDWord(counter)
    hbCommand.addWord(fraction)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param mode increment: 1 = Zaehler inkrementieren.
  @param debounceTime 0 - 255[ms].
  @param reportTime Zeitintervall in Minuten nach dem der Zaehler den aktuellen Stand meldet.
  @param scaleFaktor Anzahl Impulse pro Einheit z.B. pro kWh.
  """
  def Configuration(self, mode:MMode, debounceTime:int, reportTime:int, scaleFaktor:int):
    LOGGER.debug("Configuration"+" mode = "+str(mode)+" debounceTime = "+str(debounceTime)+" reportTime = "+str(reportTime)+" scaleFaktor = "+str(scaleFaktor))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(mode.getValue())
    hbCommand.addByte(debounceTime)
    hbCommand.addWord(reportTime)
    hbCommand.addWord(scaleFaktor)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param counter Zaehler ganze Einheiten.
  @param fraction Bruchteil 0-scaleFactor.
  """
  def Status(self, counter:int, fraction:int):
    LOGGER.debug("Status"+" counter = "+str(counter)+" fraction = "+str(fraction))
    hbCommand = HausBusCommand(self.objectId, 129, "Status")
    hbCommand.addDWord(counter)
    hbCommand.addWord(fraction)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param counter Zaehler ganze Einheiten.
  @param fraction Bruchteil 0-scaleFactor.
  """
  def evStatus(self, counter:int, fraction:int):
    LOGGER.debug("evStatus"+" counter = "+str(counter)+" fraction = "+str(fraction))
    hbCommand = HausBusCommand(self.objectId, 200, "evStatus")
    hbCommand.addDWord(counter)
    hbCommand.addWord(fraction)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")


